/* File tsize.c
 * October 10, 2002
 * By Doug Mink, Harvard-Smithsonian Center for Astrophysics
 * Send bug reports to dmink@cfa.harvard.edu
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <math.h>
#include "libtiff/tiffio.h"
#include "libwcs/fitsfile.h"

static void usage();
static void PrintSize ();

static int verbose = 0;		/* verbose/debugging flag */
static char *RevMsg = "TSIZE 3.2.0, 10 October 2002, Doug Mink (dmink@cfa.harvard.edu)";
static int version = 0;		/* If 1, print only program name and version */
static int first = 1;

main (ac, av)
int ac;
char **av;
{
    char *str;
    char *listfile;
    char filename[256];
    int ifile, nfile;
    FILE *flist;

    /* Check for help or version command first */
    str = *(av+1);
    if (!str || !strcmp (str, "help") || !strcmp (str, "-help"))
	usage();
    if (!strcmp (str, "version") || !strcmp (str, "-version")) {
	version = 1;
	usage();
	}

    /* crack arguments */
    for (av++; --ac > 0 && *(str = *av) == '-'; av++) {
	char c;
	while (c = *++str) {
	    switch (c) {
		case 'v':	/* more verbosity */
		    verbose++;
		    break;
	        default:
		    usage();
		    break;
		}
    	    }
	}

    /* now there are ac remaining file names starting at av[0] */
    if (ac == 0)
	usage ();

     else if (*av[0] == '@') {
	listfile = *av + 1;
	if (isimlist (listfile)) {
	    nfile = getfilelines (listfile);
	    if ((flist = fopen (listfile, "r")) == NULL) {
		fprintf (stderr,"I2F: Image list file %s cannot be read\n",
			 listfile);
		usage ();
		}
	    for (ifile = 0; ifile < nfile; ifile++) {
		first_token (flist, 254, filename);
		PrintSize (filename);
		}
	    fclose (flist);
	    }
	}

    else {
	while (ac-- > 0) {
	    char *fn = *av++;
	    PrintSize (fn);
	    }
	}

    return (0);
}

static void
usage ()
{
    fprintf (stderr,"%s\n",RevMsg);
    if (version)
	exit (-1);
    fprintf (stderr,"Print the dimensions of TIFF images\n");
    fprintf(stderr,"Usage: tsize [-v] file.tif ...\n");
    fprintf(stderr,"  or : tsuze [-v] @tifflist\n");
    fprintf(stderr,"  -v: verbose\n");
    exit (1);
}

static void
PrintSize (name)
char *name;
{
    char *header;	/* FITS header */
    int lhead;		/* Maximum number of bytes in FITS header */
    int nbhead;		/* Actual number of bytes in FITS header */
    char *filename;	/* Pointer to start of file name */
    char tiffname[256];	/* Name of TIFF file */
    int nbytes;
    int i, j;
    char *ext;		/* Pointer to start of extension */
    int nc, ier, bitpix;
    int tpix, bytepix, nprow;
    int nbrow, nblock, nbleft, sampix, nbout;
    uint16 nbsamp, nspix;
    uint32 nrows, ncols;
    uint32 row;
    int irow;
    int vpix;
    TIFF *tif;

    /* Open TIFF image */
    tif = TIFFOpen (name, "r");
    if (tif) {
	filename = strrchr (name,'/');
	if (filename)
	    filename = filename + 1;
	else
	    filename = name;
	if (verbose && first) {
	    fprintf (stderr,"%s\n",RevMsg);
	    fprintf (stderr, "Read TIFF image file %s", name);
	    }
	}

    /* Add .tif extension to make TIFF header file name if not present */
    else {
	strcpy (tiffname, name);
	strcat (tiffname,".tif");
	tif = TIFFOpen (tiffname, "r");
	if (!tif) {
	    strcat (tiffname,"f");
	    tif = TIFFOpen (tiffname, "r");
	    }
	if (!tif) {
	    fprintf (stderr,"Cannot read TIFF image file %s\n", tiffname);
	    return;
	    }
	filename = strrchr (tiffname,'/');
	if (filename)
	    filename = filename + 1;
	else
	    filename = tiffname;
	if (verbose && first) {
	    fprintf (stderr,"%s\n",RevMsg);
	    fprintf (stderr,"Read TIFF image file %s\n", tiffname);
	    }
	}

    /* Get pixel size (negate for unsigned) */
    TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &nbsamp);
    bitpix = nbsamp;
    bytepix = nbsamp / 8;

    /* Set samples per pixel */
    TIFFGetField (tif, TIFFTAG_SAMPLESPERPIXEL, &nspix);
    sampix = nspix;

    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &ncols);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &nrows);

    if (sampix > 1)
	printf ("%s: %d x %d , %d bits, %d samples per pixel\n",
		filename, nrows, ncols, bitpix, sampix);
    else
	printf ("%s: %d x %d , %d bits, %d sample per pixel\n",
		filename, nrows, ncols, bitpix, sampix);

    TIFFClose(tif);

    return;
}

/* Sep 30 2002	New program based on t2f
 * Oct 10 2002	Print 'sample' if only one
 */
